"""
A command line tool to edit text files encrypted with GnuPG whilst
preventing the plaintext from being written to the hard drive.
"""
__author__ = 'Matt Deacalion Stevens'
__version__ = '0.1.1'

import getpass
import hashlib
import os
import shutil
import subprocess
import uuid
from pathlib import Path


class Mikla:
    def __init__(self, **kwargs):
        self.encrypted = kwargs.get('<encrypted-file>')
        self.tmpfs = kwargs.get('--tmpfs')
        self.editor = kwargs.get('--editor')

        if self.editor == '$EDITOR':
            self.editor = os.environ['EDITOR']

    def run(self):
        """
        Call to run Mikla.
        """
        password = getpass.getpass('Enter password: ')
        plaintext = self.decrypt(password)

        if self.launch_editor(plaintext):
            self.encrypt(password, plaintext)

        os.unlink(plaintext)

    def launch_editor(self, plain, editor=None):
        """
        Takes a plaintext path and launches the text editor. Returns
        `True` if the plaintext file was changed while the text
        editor was running.
        """
        if editor is None:
            editor = self.editor

        before_checksum = self.checksum(plain)

        subprocess.run([editor, plain])

        return before_checksum != self.checksum(plain)

    def checksum(self, text_file):
        """
        Takes a path to a text file and returns a string containing
        the SHA1 checksum digest for it.
        """
        hasher = hashlib.sha1()
        hasher.update(open(text_file).read().encode())

        return hasher.hexdigest()

    def encrypt(self, password, plain, encrypted=None):
        """
        Takes a password string, a plaintext file path and an
        encrypted file path. Then encrypts the plaintext file to the
        encrypted path.
        """
        if encrypted is None:
            encrypted = self.encrypted

        backup = '{}.bak'.format(encrypted)
        shutil.move(encrypted, backup)

        completed_process = subprocess.run(
            [
                'gpg',
                '--symmetric',
                '--armor',
                '--batch',
                '--passphrase',
                password,
                '--output',
                encrypted,
                plain,
            ],
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )

        if completed_process.returncode == 0:
            os.unlink(backup)
        else:
            shutil.move(backup, encrypted)

            raise RuntimeError(
                'Encryption of "{}" failed'.format(plain),
            )

    def decrypt(self, password, encrypted=None, tmpfs=None):
        """
        Takes a password string, an encrypted file path and a
        temporary filesystem path. Then decrypts the encrypted file
        using GnuPG. Returns the path to a file containing the
        plaintext.
        """
        if encrypted is None:
            encrypted = self.encrypted

        if tmpfs is None:
            tmpfs = self.tmpfs

        # do all of the checking manually,
        # so we don't pointlessly run GnuPG
        if not self.gpg_exists():
            raise FileNotFoundError('GnuPG not installed')

        if not os.access(encrypted, os.F_OK):
            raise FileNotFoundError(
                'File not found: {}'.format(encrypted),
            )

        if not os.access(encrypted, os.R_OK):
            raise FileNotFoundError(
                'File not readable: {}'.format(encrypted),
            )

        if not os.access(tmpfs, os.W_OK):
            raise FileNotFoundError(
                'tmpfs directory not writable: {}'.format(tmpfs),
            )

        plain = self.get_available_file_path(tmpfs)

        completed_process = subprocess.run(
            [
                'gpg',
                '--decrypt',
                '--batch',
                '--passphrase',
                password,
                '--output',
                plain,
                encrypted,
            ],
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )

        if completed_process.returncode != 0:
            raise RuntimeError(
                'Decryption of "{}" failed, '
                'was the password correct?'.format(encrypted),
            )

        return plain

    def gpg_exists(self, gpg_executable='gpg'):
        """
        Returns `True` if GnuPG is installed, `False` if not.
        """
        return bool(shutil.which(gpg_executable))

    def get_available_file_path(self, directory):
        """
        Takes a directory string and returns an available file path.
        """
        while True:
            filepath = Path(directory) / str(uuid.uuid4())

            if not filepath.exists():
                return str(filepath)



# coding=utf-8

"""Usage:
  mikla [--editor=<editor>] [--tmpfs=<path>] <encrypted-file>
  mikla (-h | --help | --version)

Options:
  --version                   show program's version number and exit.
  -h, --help                  show this help message and exit.
  -e, --editor=<editor>       specify the editor to use [default: $EDITOR].
  -t, --tmpfs=<path>          the path to a temporary file system [default: /dev/shm].
"""
import sys

from docopt import docopt

from mikla import Mikla, __version__


def main():
    try:
        Mikla(**docopt(__doc__, version=__version__)).run()
    except (FileNotFoundError, RuntimeError) as error:
        print(error, file=sys.stderr)
        sys.exit(1)

if __name__ == '__main__':
    main()



from setuptools import setup

import mikla

setup(
    name='mikla',
    version=mikla.__version__.strip(),
    url='http://dirtymonkey.co.uk/mikla',
    license='MIT',
    author=mikla.__author__.strip(),
    author_email='matt@dirtymonkey.co.uk',
    description=mikla.__doc__.strip().replace('\n', ' '),
    long_description=open('README.rst').read(),
    keywords='encryption security gnupg gpg',
    packages=['mikla'],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'mikla = mikla.main:main',
        ],
    },
    install_requires=[
        'docopt>=0.6.2',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Operating System :: POSIX :: BSD',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Security :: Cryptography',
        'Topic :: Communications',
        'Topic :: Utilities',
    ],
)
